/*
 * Bermuda Syndrome engine rewrite
 * Copyright (C) 2007-2011 Gregory Montoir
 */

#include "game.h"
#include "systemstub.h"

#ifdef WII
#include <gccore.h>
#include <ogcsys.h>
#include "fat.h"
#include <sys/stat.h>
#define deviceSD "sd:"
#define deviceUSB "usb:"
#endif



static SystemStub *g_stub;

static const char *USAGE =
	"Bermuda Syndrome\n"
	"Usage: bs [OPTIONS]...\n"
	"  --datapath=PATH   Path to data files (default 'DATA')\n"
	"  --savepath=PATH   Path to save files (default '.')\n"
	"  --musicpath=PATH  Path to music files (default 'MUSIC')\n";

static bool parseOption(const char *arg, const char *longCmd, const char **opt) {
	bool handled = false;
	if (arg[0] == '-' && arg[1] == '-') {
		if (strncmp(arg + 2, longCmd, strlen(longCmd)) == 0) {
			*opt = arg + 2 + strlen(longCmd);
			handled = true;
		}
	}
	return handled;
}

static void exitMain() {
	if (g_stub) {
		g_stub->destroy();
		delete g_stub;
		g_stub = 0;
	}
}

#undef main
//int main(int argc, char *argv[]) {
extern "C" int SDL_main(int argc, char **argv) {

#ifdef WII
	fatInit(16, 1);
	struct stat st;
	bool device = true;

	/* Checking if game's datas exist : USB first then SD */
	if(stat("usb:/bermuda/data/jack/",&st) == 0)
	{
		printf("\nLoading usb:/bermuda/ ...\n");
		SDL_Delay(500); 
		device=true;
	}
	else if(stat("sd:/bermuda/data/jack/",&st) == 0)
	{
		printf("\nLoading sd:/bermuda/ ...\n");
		SDL_Delay(500); 
		device=false;
	}
	else
	{
		printf("\nNo game data found !\n");
		printf("Check your sd:/bermuda or usb:/bermuda folder.\n");
		printf("Exiting ...");
		SDL_Delay(2000);
		exit(1);
		
	}
	const char *dataPath= (device) ? deviceUSB"/bermuda/data" : deviceSD"/bermuda/data";
	const char *savePath= (device) ? deviceUSB"/bermuda/save" : deviceSD"/bermuda/save";
	const char *musicPath= (device) ? deviceUSB"/bermuda/music" : deviceSD"/bermuda/music";
#else
	const char *dataPath = "DATA";
	const char *savePath = ".";
	const char *musicPath = "MUSIC";
#endif

	for (int i = 1; i < argc; ++i) {
		bool opt = false;
		if (strlen(argv[i]) >= 2) {
			opt |= parseOption(argv[i], "datapath=", &dataPath);
			opt |= parseOption(argv[i], "savepath=", &savePath);
			opt |= parseOption(argv[i], "musicpath=", &musicPath);
		}
		if (!opt) {
			printf("%s", USAGE);
			return 0;
		}
	}
	g_debugMask = DBG_INFO; // | DBG_GAME | DBG_OPCODES | DBG_DIALOGUE;
	g_stub = SystemStub_SDL_create();
	atexit(exitMain);
	Game *g = new Game(g_stub, dataPath, savePath, musicPath);
	g->mainLoop();
	delete g;
	fatUnmount("sd:");
    fatUnmount("usb:");
	return 0;
}
