#include "read_entry.h"

#include <cstdio>
#include <stdlib.h>
#include <assert.h>

#define LITTLE_ENDIAN


#ifdef LITTLE_ENDIAN

static inline unsigned short bswap_16(unsigned short x) {
  return (x>>8) | (x<<8);
}

static inline unsigned int bswap_32(unsigned int x) {
  return (bswap_16(x&0xffff)<<16) | (bswap_16(x>>16));
}

static inline unsigned long long bswap_64(unsigned long long x) {
  return (((unsigned long long)bswap_32(x&0xffffffffull))<<32) | (bswap_32(x>>32));
}
#else
#define bswap_16(x) (x)
#define bswap_32(x) (x)
#define bswap_64(x) (x)
#endif

void swap_packet(packet * in, packet * out)
{
	out->ptr = bswap_32(in->ptr);
	out->arg1 = bswap_32(in->arg1);
	out->arg2 = bswap_32(in->arg2);
	out->line = bswap_32(in->line);
	out->strsize = bswap_32(in->strsize);
	out->type = bswap_32(in->type);
};

bool read_entry(FILE * file, memtrace_entry *entry)
{
	packet native;
	if(fread(&native, sizeof(packet), 1, file) != 1) return false;

	swap_packet(&native, &entry->p);

	char * buf = new char[entry->p.strsize+1];

	if(fread(buf, entry->p.strsize, 1, file) != 1)
	{
		delete [] buf;
		return false;
	}

	buf[entry->p.strsize] = 0;

	entry->file = buf;

	delete [] buf;

	return true;
}

#include <zlib.h>

z_stream comp_stream;
bool comp_inited = false;
unsigned char inbuf[1000];
uint64_t base_tick = 0;

bool read_stream(FILE *file, void * p, size_t s)
{
	comp_stream.next_out = (Bytef*)p;
	comp_stream.avail_out = s;

	while(comp_stream.avail_out > 0)
	{
		if(comp_stream.avail_in == 0)
		{
			size_t byte_read = fread(inbuf, 1, sizeof(inbuf), file);
			comp_stream.avail_in = byte_read;
			comp_stream.next_in = inbuf;
		}
		
		int err = inflate(&comp_stream, Z_NO_FLUSH);

		if(comp_stream.avail_in == 0 && feof(file))
		{
			int err = inflate(&comp_stream, Z_FINISH);
			inflateEnd(&comp_stream);
			return false;
		}
	}
	return true;
}

bool read_entry_zlib(FILE *file, memtrace_entry *entry)
{
	if(!comp_inited)
	{
		comp_stream.zalloc = NULL;
		comp_stream.zfree = NULL;
		comp_stream.opaque = NULL;
		comp_stream.next_in = inbuf;
		comp_stream.avail_in = 0;
		comp_stream.next_out = NULL;
		comp_stream.avail_out = 0;

		assert(inflateInit(&comp_stream) == Z_OK);

		comp_inited = true;
	}

	packet p;

	comp_stream.next_out = (Bytef*)&p;
	comp_stream.avail_out = sizeof(packet);

	if(!read_stream(file, (void*)&p, sizeof(packet))) return false;

	swap_packet(&p, &entry->p);

	char * buf = new char[entry->p.strsize+1];

	if(!read_stream(file, (void *)buf, entry->p.strsize))
	{
		delete [] buf;
		return false;
	}

	buf[entry->p.strsize] = 0;

	entry->file = buf;

	delete [] buf;


	return true;
}
