#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <cstdio>
#include "read_entry.h"
#include <vector>
#include <assert.h>
#include <set>
#include <algorithm>
#include <sstream>

#define MAJOR_VER 0
#define MINOR_VER 1

void print_usage()
{
	printf(" memtrace_parse v%d.%d by litghost\n", MAJOR_VER, MINOR_VER);
	printf("    Usage:\n");
	printf("       memtrace_parse [-x] <input filename>\n\n");
	printf("      -x Input file is compressed with zlib, decompress before analysis\n");
}
	
enum ArgFlags
{
	NO_FLAG,
	EXTRACT
};

int main(int argc, const char *argv[])
{
	if(argc < 2)
	{
		print_usage();
		return EXIT_FAILURE;
	}
	
	const char ** argv_end = argv+argc;
	
	++argv;
	
	typedef std::map<std::string,ArgFlags> argmap_t;
	argmap_t argmap;
	
	bool extract = false;
	const char *binary_src = NULL;
	const char *input_file = NULL;
	const char *output_file = NULL;
	
	argmap["-x"] = EXTRACT;

	std::map<packetType, std::string> nameMap;

	nameMap[MEMTRACE_NONE] = "unknown";
	nameMap[MEMTRACE_MALLOC] = "malloc";
	nameMap[MEMTRACE_MEMALIGN] = "memalign";
	nameMap[MEMTRACE_CALLOC] = "calloc";
	nameMap[MEMTRACE_REALLOC] = "realloc";
	nameMap[MEMTRACE_FREE] = "free";

	while(argv != argv_end)
	{
		switch(argmap[*argv])
		{
			case EXTRACT:
				extract = true;
				break;
			case NO_FLAG:
			default:
				if(input_file == NULL)
				{
					input_file = *argv;
				}
				else
				{
					print_usage();
					fprintf(stderr, "\n\tProvide only one input file\n");
					exit(EXIT_FAILURE);
				}
				break;			
		}
		
		
		// Next argument
		++argv;
	}

	if(input_file == NULL)
	{
		print_usage();
		fprintf(stderr, "\n\tYou must provide an input file\n");
		exit(EXIT_FAILURE);
	}

	FILE * f = fopen(input_file, "rb");

	if(f == NULL)
	{
		fprintf(stderr, "\n\tFile %s could not be opened\n", input_file);
		exit(EXIT_FAILURE);
	}

	typedef bool (*read_fun_t)(FILE * file, memtrace_entry *);

	read_fun_t read_fun;

	if(extract)
	{
		read_fun = &read_entry_zlib;
	}
	else
	{
		read_fun = &read_entry;
	}

	memtrace_entry ent;

	fprintf(stdout, "Ret\tFunc\tArg1\tArg1\tFile\tLine\n");

	while(read_fun(f, &ent))
	{
		fprintf(stdout, "%u\t%s\t%u\t%u\t%s\t%d\n", ent.p.ptr, nameMap[(packetType)ent.p.type].c_str(), ent.p.arg1, ent.p.arg2, ent.file.c_str(), ent.p.line);
	}

	fclose(f);
	fflush(stdout);
}
